import { HttpException, HttpService, Injectable, HttpStatus } from '@nestjs/common';
import { Logger } from '@nestjs/common';
import { Observable } from 'rxjs';
import { AxiosResponse } from 'axios';
import { Cron, CronExpression } from '@nestjs/schedule';
import * as _ from 'lodash';
import * as https from 'https';

@Injectable()
export class ApticStructuresService {
  constructor(private readonly httpService: HttpService) {}

  public formatApticStructures(postalCodeData: any[]): any {
    // Get all postal code in one array
    const postalCodeArray = _.flatten(
      postalCodeData.map((data) => {
        return data.codesPostaux;
      })
    );

    // Call APTIC Api's
    const postalCodePromises = postalCodeArray.map((postalCode) => {
      return this.getApticStructures(postalCode).toPromise();
    });

    Promise.all(postalCodePromises).then((data) => {
      const structuresData = _.flatten(
        data.map((tmp: { data }) => {
          return tmp.data.data;
        })
      );
      console.log(structuresData);
      //TODO: create structure
      //TODO: clean with already existing structures
    });
  }

  @Cron(CronExpression.EVERY_MINUTE)
  public async getMetopoleMunicipality() {
    const req =
      'https://download.data.grandlyon.com/ws/grandlyon/adr_voie_lieu.adrcomgl/all.json?maxfeatures=-1&start=1';
    Logger.log(`Request : ${req}`, 'ApticStructuresService - getMetopoleMunicipality');
    this.httpService.get(encodeURI(req)).subscribe(
      (data) => {
        const inseeArray = data.data.values.map((municipality) => {
          return this.getPostalCodeWithINSEE(municipality.insee).toPromise();
        });
        Promise.all(inseeArray).then((inseData) => {
          const postalCodeArray = inseData.map((cpData: { data; config }) => {
            return cpData.data;
          });
          this.formatApticStructures(postalCodeArray);
        });
      },
      (err) => Logger.error(err)
    );
  }

  public getPostalCodeWithINSEE(inseeCode: string): Observable<AxiosResponse<any>> {
    const req = `https://geo.api.gouv.fr/communes/${inseeCode}?fields=codesPostaux&format=json`;
    Logger.log(`Request : ${req}`, 'ApticStructuresService - getMetopoleMunicipality');
    return this.httpService.get(encodeURI(req));
  }

  public getApticStructures(postalCodeData: string): Observable<AxiosResponse<any>> {
    const agent = new https.Agent({
      rejectUnauthorized: false,
    });
    const req = `https://presence.aptic.fr/postal_code/${postalCodeData}`;
    Logger.log(`Request : ${req}`, 'ApticStructuresService');
    return this.httpService.get(req, {
      httpsAgent: agent,
      headers: {
        api_key: process.env.APTIC_TOKEN,
      },
    });
  }
}
